using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Media;
using Microsoft.Xna.Framework.Net;
using Microsoft.Xna.Framework.Storage;


namespace Ping_Pong
{
    /// <summary>
    /// This is a game component that implements IUpdateable.
    /// </summary>
    public class Ball : Microsoft.Xna.Framework.DrawableGameComponent
    {
        //Declare class-wide variables
        GraphicsDevice graphics;
        SpriteBatch spriteBatch;

	    //collision rectangles
        public Rectangle all = new Rectangle();

        //"Speed Overload" message
        bool SpeedOverload = false;
        int SpeedOverloadCnt = 0;
        const int SPEED_OVERLOAD_MAX_CNT = 50;

	    //properties
        private Texture2D _skin;
        public Texture2D Texture
        {
            get
            {
                return _skin;
            }
            set
            {
                _skin = value;
            }
        }
        private Vector2 _spritePosition = new Vector2(0.0f, 250.0f);
        public Vector2 Position
        {
            get
            {
                return _spritePosition;
            }
            set
            {
                _spritePosition = value;
            }
        }
        private Vector2 _spriteSpeed = new Vector2(50.0f, 50.0f);
        public Vector2 Speed
        {
            get
            {
                return _spriteSpeed;
            }
            set
            {
                _spriteSpeed = value;
            }
        }
        private Paddle[] _paddle;
        public Paddle[] paddle
        {
            get
            {
                return _paddle;
            }
            set
            {
                _paddle = value;
            }
        }
        private SpriteFont _Arial;
        public SpriteFont Arial
        {
            get
            {
                return _Arial;
            }
            set
            {
                _Arial = value;
            }
        }
        private SpriteFont _ArialLarge;
        public SpriteFont ArialLarge
        {
            get
            {
                return _ArialLarge;
            }
            set
            {
                _ArialLarge = value;
            }
        }
        private bool _enabled = true;
        public bool enabled
        {
            get
            {
                return _enabled;
            }
            set
            {
                _enabled = value;
            }
        }
        private int[] _Score;
        public int[] Score
        {
            get
            {
                return _Score;
            }
            set
            {
                _Score = value;
            }
        }


        /// <summary>
        /// this is the constructor
        /// </summary>
        /// <param name="game"></param>
        public Ball(Game game)
            : base(game)
        {
            // TODO: Construct any child components here
        }

        /// <summary>
        /// Allows the game component to perform any initialization it needs to before starting
        /// to run.  This is where it can query for any required services and load content.
        /// </summary>
        public override void Initialize()
        {
            base.Initialize();

            //Initialize the graphics manager
            graphics = GraphicsDevice;

            // Create a new SpriteBatch, which can be used to draw textures.
            spriteBatch = new SpriteBatch(GraphicsDevice);

            //Set up the rectangle that contains the entire ball
            all.Width = _skin.Width;
            all.Height = _skin.Height;
            all.X = (int)_spritePosition.X;
            all.Y = (int)_spritePosition.Y;
        }

        /// <summary>
        /// Allows the game component to update itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        public override void Update(GameTime gameTime)
        {
            if (enabled)
            {
                //reset the "all" rectangle
                all.X = (int)_spritePosition.X;
                all.Y = (int)_spritePosition.Y;

                // Check for collisions
                if (Collision(all, paddle[0].all) || Collision(all, paddle[1].all))
                {
                    if (Collision(all, paddle[0].CollisionBoxes[0]) ||
                        Collision(all, paddle[0].CollisionBoxes[3]) ||
                        Collision(all, paddle[1].CollisionBoxes[0]) ||
                        Collision(all, paddle[1].CollisionBoxes[3]))
                    {
                        _spriteSpeed.X *= -1.25f;
                        _spriteSpeed.Y *= -1.75f;
                    }
                    else if (Collision(all, paddle[0].CollisionBoxes[1]) ||
                        Collision(all, paddle[0].CollisionBoxes[2]) ||
                        Collision(all, paddle[1].CollisionBoxes[1]) ||
                        Collision(all, paddle[1].CollisionBoxes[2]))
                    {
                        if (paddle[0].Speed.Y == 0 || paddle[1].Speed.Y == 0)
                            _spriteSpeed.X *= -1f;
                        if (paddle[0].Speed.Y > 0 || paddle[1].Speed.Y > 0)
                            _spriteSpeed.X *= 0.5f;
                    }
                }

                if (_spriteSpeed.X > 600 || _spriteSpeed.Y > 600)
                {
                    //speed overload. reset speed to normal
                    SpeedOverload = true;
                    _spriteSpeed.X = 50.0f;
                    _spriteSpeed.Y = 50.0f;
                }

                // Move the sprite around.
                UpdateSprite(gameTime);
            }

            base.Update(gameTime);
        }

        protected bool Collision(Rectangle b1, Rectangle b2)
        {
            return b1.Intersects(b2);
        }
        
        void UpdateSprite(GameTime gameTime)
        {
            //define the max/min values of x and y
            int MaxX =
                graphics.Viewport.Width - _skin.Width;
            int MinX = 0;
            int MaxY =
                graphics.Viewport.Height - _skin.Height;
            int MinY = 0;

            // Check for bounce.
            if (_spritePosition.X > MaxX + _skin.Width + 5)
            {
                Score[1]++;
                _spriteSpeed.X *= -1;
                _spritePosition.X = MaxX / 2;
                _spritePosition.Y = MaxY / 2;
            }

            else if (_spritePosition.X < MinX - _skin.Width - 5)
            {
                Score[2]++;
                _spriteSpeed.X *= -1;
                _spritePosition.X = MaxX / 2;
                _spritePosition.Y = MaxY / 2;
            }

            if (_spritePosition.Y > MaxY)
            {
                _spriteSpeed.Y *= -1;
                _spritePosition.Y = MaxY;
            }

            else if (_spritePosition.Y < MinY)
            {
                _spriteSpeed.Y *= -1;
                _spritePosition.Y = MinY;
            }

            // Move the sprite by speed, scaled by elapsed time.
            _spritePosition += _spriteSpeed * (float)0.05;
        }

        /// <summary>
        /// This is called when the game should draw itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        public override void Draw(GameTime gameTime)
        {
            string tempString = "Player 1's Score";
            // define some size values
            float LeftAlign = paddle[0].Position.X + paddle[0].Texture.Width + paddle[0].Position.X;
            float RightAlign = graphics.Viewport.Width - (paddle[0].Position.X + paddle[0].Texture.Width + (tempString.Length - 1) * 12);

            // Draw everything.
            spriteBatch.Begin(SpriteBlendMode.AlphaBlend);
            //draw the text
            spriteBatch.DrawString(Arial, "Player 1's Score", new Vector2(LeftAlign, 10.0f), Color.Black);
            spriteBatch.DrawString(ArialLarge, Score[1].ToString(), new Vector2(LeftAlign, 30.0f), Color.Black);
            spriteBatch.DrawString(Arial, "Player 2's Score", new Vector2(RightAlign, 10.0f), Color.Black);
            spriteBatch.DrawString(ArialLarge, Score[2].ToString(), new Vector2(RightAlign + 115.0f, 30.0f), Color.Black);

            if (SpeedOverload && SpeedOverloadCnt < SPEED_OVERLOAD_MAX_CNT)
            {
                spriteBatch.DrawString(ArialLarge, "SPEED OVERLOAD", new Vector2(LeftAlign * 2.5f, (graphics.Viewport.Height / 2) - 12), Color.Black);
                SpeedOverloadCnt ++;
            }
            if (SpeedOverloadCnt == SPEED_OVERLOAD_MAX_CNT)
            {
                SpeedOverload = false;
                SpeedOverloadCnt = 0;
            }
            spriteBatch.Draw(_skin, _spritePosition, Color.White);
            spriteBatch.End();

            base.Draw(gameTime);
        }
    }
}