using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Media;
using Microsoft.Xna.Framework.Net;
using Microsoft.Xna.Framework.Storage;

namespace Ping_Pong
{
    /// <summary>
    /// This is the main type for your game
    /// </summary>
    public class Game1 : Microsoft.Xna.Framework.Game
    {
        //Declare class-wide variables
        Paddle[] paddle = new Paddle[2];
        Ball ball;
        SpriteFont Arial;
        SpriteFont ArialLarge;
        int[] Score = new int[3];
        String BackgroundImage = @"textures\background";
        Texture2D background;
        Rectangle mainFrame;
        Texture2D BallTexture;
        Texture2D PaddleTexture;
        // Set the coordinates to draw the sprite at.
        Vector2 spritePosition = Vector2.Zero;
        Vector2 spritePosition2 = new Vector2(30.0f, 250.0f);
        Vector2 spritePosition3 = new Vector2(100.0f, 250.0f);
        // Store some information about the sprite's motion.
        Vector2 spriteSpeed = new Vector2(50.0f, 50.0f);
        Vector2 spriteSpeed2 = new Vector2(0.0f, 0.0f);
        Vector2 spriteSpeed3 = new Vector2(0.0f, 0.0f);
        //Pause variables
        bool Paused = false;
        string PauseMessage = "Press \"Pause\" to continue";
        KeyboardState OldState;
        int pauseDelay = 0;
        //Game over sounds
        bool GameOver = false;

        public GraphicsDeviceManager graphics;
        SpriteBatch spriteBatch;

        public Game1()
        {
            graphics = new GraphicsDeviceManager(this);
            Content.RootDirectory = "Content";

            //set up the paddles
            paddle[0] = new Paddle(this);
            paddle[1] = new Paddle(this);

            //set up the ball
            ball = new Ball(this);

            //Add components
            Components.Add(paddle[0]);
            Components.Add(paddle[1]);
            Components.Add(ball);
        }


        /// <summary>
        /// Allows the game to perform any initialization it needs to before starting to run.
        /// This is where it can query for any required services and load any non-graphic
        /// related content.  Calling base.Initialize will enumerate through any components
        /// and initialize them as well.
        /// </summary>
        protected override void Initialize()
        {
            LoadContent();

            // Set up the oldState
            KeyboardState oldState = Keyboard.GetState();

            base.Initialize();
        }

        /// <summary>
        /// LoadContent will be called once per game and is the place to load
        /// all of your content.
        /// </summary>

        protected override void LoadContent()
        {
            // load the score array
            Score[0] = 5; //Max score;
            Score[1] = 0; //Player one's score
            Score[2] = 0; //Player two's score

            // Load the background
            background = Content.Load<Texture2D>(BackgroundImage);
            mainFrame = new Rectangle(0, 0, GraphicsDevice.Viewport.Width, GraphicsDevice.Viewport.Height);

            //load the fonts
            Arial = Content.Load<SpriteFont>(@"font\Arial");
            ArialLarge = Content.Load<SpriteFont>(@"font\ArialLarge");

            // Create a new SpriteBatch, which can be used to draw textures.
            spriteBatch = new SpriteBatch(GraphicsDevice);
            BallTexture = Content.Load<Texture2D>(@"textures\smiley");
            PaddleTexture = Content.Load<Texture2D>(@"textures\paddle");

            //move the sprite to the middle
            spritePosition.X = (graphics.GraphicsDevice.Viewport.Width - BallTexture.Width) / 2;
            spritePosition.Y = (graphics.GraphicsDevice.Viewport.Height - BallTexture.Height) / 2;

            //move the paddles to their respected side of the screen
            spritePosition2.Y = spritePosition3.Y = graphics.GraphicsDevice.Viewport.Height / 3;
            spritePosition3.X = graphics.GraphicsDevice.Viewport.Width - (spritePosition2.X + PaddleTexture.Width);
            
            //set up the paddles
            paddle[0].Texture = PaddleTexture;
            paddle[0].Position = spritePosition2;
            paddle[0].Speed = spriteSpeed2;
            paddle[0].RightOrLeft = 1;
            paddle[1].Texture = PaddleTexture;
            paddle[1].Position = spritePosition3;
            paddle[1].Speed = spriteSpeed3;
            paddle[1].RightOrLeft = 0;

            //set up the ball
            ball.Texture = BallTexture;
            ball.Position = spritePosition;
            ball.Speed = spriteSpeed;
            ball.paddle = paddle;
            ball.Arial = Arial;
            ball.ArialLarge = ArialLarge;
            ball.Score = Score;
        }

        /// <summary>
        /// UnloadContent will be called once per game and is the place to unload
        /// all content.
        /// </summary>
        protected override void UnloadContent()
        {
            // TODO: Unload any non ContentManager content here
        }

        /// <summary>
        /// Allows the game to run logic such as updating the world,
        /// checking for collisions, gathering input, and playing audio.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Update(GameTime gameTime)
        {
            //temp keystate
            KeyboardState tempState = Keyboard.GetState();
            
            //test for game over
            if (GameOver && Paused)
            {
                GameOver = false;
                ResetGame();
            }
            if(!GameOver && !Paused)
            {
                //reset the pause message once game begins again
                PauseMessage = "Press \"Pause\" to continue";
            }

            //check for end game
            if (Score[1] == Score[0] || Score[2] == Score[0])
            {
                PauseMessage = "Game Over. Please press \"Space\" to continue";
                Paused = true;
                GameOver = true;
            }

            // Allows the game to exit
            if (GamePad.GetState(PlayerIndex.One).Buttons.Back ==
                ButtonState.Pressed || tempState.IsKeyDown(Keys.Escape))
                this.Exit();

            // Allows game to be reset
            if (tempState.IsKeyDown(Keys.R))
                if (!OldState.IsKeyDown(Keys.R))
                    ResetGame();

            // Allows game to be paused
            if (tempState.IsKeyDown(Keys.Space))
                if (!OldState.IsKeyDown(Keys.Space))
                {
                    if (pauseDelay == 0)
                        Paused = true;
                    else
                        Paused = false;
                }

            if (pauseDelay == 1)
            {
                PauseGame();
            }

            OldState = tempState;

            base.Update(gameTime);
        }

        /// <summary>
        /// This is called when the game should draw itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Draw(GameTime gameTime)
        {
            graphics.GraphicsDevice.Clear(Color.CornflowerBlue);

            string tempString = "Player 1's Score";
            // define some size values
            float LeftAlign = spritePosition2.X + PaddleTexture.Width + spritePosition2.X;
            float RightAlign = graphics.GraphicsDevice.Viewport.Width - (spritePosition2.X + PaddleTexture.Width + (tempString.Length - 1) * 12);

            // Draw everything.
            spriteBatch.Begin(SpriteBlendMode.AlphaBlend);
            spriteBatch.Draw(background, mainFrame, Color.White);
            //draw the text
            spriteBatch.DrawString(Arial, "Player 1 Controls: \nW = Up, S = Down", new Vector2(LeftAlign, graphics.GraphicsDevice.Viewport.Height - 60.0f), Color.Black);
            spriteBatch.DrawString(Arial, "Player 2 Controls: \nUp Arrow = Up, \nDown Arrow = Down", new Vector2(RightAlign - 30.0f, graphics.GraphicsDevice.Viewport.Height - 80.0f), Color.Black);
            spriteBatch.DrawString(Arial, "ESC = quit, \nSpace Bar = Pause, \nR = Reset", new Vector2(LeftAlign * 4, graphics.GraphicsDevice.Viewport.Height - 80.0f), Color.DarkGray);
            if (Paused)
            {
                spriteBatch.Draw(background, mainFrame, new Color(150, 150, 150, (byte)180));
                spriteBatch.DrawString(Arial, PauseMessage, new Vector2(LeftAlign * 2.5f, (graphics.GraphicsDevice.Viewport.Height / 2) + 12), Color.Black);

                pauseDelay++;
            }
            spriteBatch.End();

            base.Draw(gameTime);
        }

        protected void ResetGame()
        {
            // reset the score array
            Score[1] = 0; //Player one's score
            Score[2] = 0; //Player two's score

            //move the sprite to the middle
            spritePosition.X = (graphics.GraphicsDevice.Viewport.Width - BallTexture.Width) / 2;
            spritePosition.Y = (graphics.GraphicsDevice.Viewport.Height - BallTexture.Height) / 2;

            //move the paddles to their respected side of the screen
            spritePosition2.Y = spritePosition3.Y = graphics.GraphicsDevice.Viewport.Height / 3;
            spritePosition3.X = graphics.GraphicsDevice.Viewport.Width - (spritePosition2.X + PaddleTexture.Width);

            // Store some information about the sprite's motion.
            spriteSpeed.X = spriteSpeed.Y = 50.0f;
            spriteSpeed2.X = spriteSpeed2.Y = spriteSpeed3.X = spriteSpeed3.Y = 0.0f;

            //rest the paddles
            paddle[0].Position = spritePosition2;
            paddle[0].Speed = spriteSpeed2;
            paddle[1].Position = spritePosition3;
            paddle[1].Speed = spriteSpeed3;

            //rest the ball
            ball.Position = spritePosition;
            ball.Speed = spriteSpeed;
        }

        protected void PauseGame()
        {
            if (pauseDelay == 1)
            {
                if (Paused)
                {
                    ball.enabled = false;
                    paddle[0].enabled = false;
                    paddle[1].enabled = false;
                }
                else
                {
                    ball.enabled = true;
                    paddle[0].enabled = true;
                    paddle[1].enabled = true;
                }
                pauseDelay = 0;
            }
            else if(!Paused)
            {
                ball.enabled = true;
                paddle[0].enabled = true;
                paddle[1].enabled = true;
            }

            if (GameOver)
            {
                GameOver = false;
                Paused = false;
            }
        }
    }
}
