using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Media;
using Microsoft.Xna.Framework.Net;
using Microsoft.Xna.Framework.Storage;


namespace Ping_Pong
{
    /// <summary>
    /// This is a game component that implements IUpdateable.
    /// </summary>
    public class Paddle : Microsoft.Xna.Framework.DrawableGameComponent
    {
        //Class-wide variables
        private KeyboardState oldState;
        GraphicsDevice graphics;
        SpriteBatch spriteBatch;

        //properties
        private Texture2D _skin;
        public Texture2D Texture
        {
            get
            {
                return _skin;
            }
            set
            {
                _skin = value;
            }
        }
        private Vector2 _spritePosition = new Vector2(0.0f, 250.0f);
        public Vector2 Position
        {
            get
            {
                return _spritePosition;
            }
            set
            {
                _spritePosition = value;
            }
        }
        private Vector2 _spriteSpeed = new Vector2(0.0f, 0.0f);
        public Vector2 Speed
        {
            get
            {
                return _spriteSpeed;
            }
            set
            {
                _spriteSpeed = value;
            }
        }
        private int _RightOrLeft; // 1 = left, 0 = right
        public int RightOrLeft
        {
            get
            {
                return _RightOrLeft;
            }
            set
            {
                _RightOrLeft = value;
            }
        }
        private bool _enabled = true;
        public bool enabled
        {
            get
            {
                return _enabled;
            }
            set
            {
                _enabled = value;
            }
        } 

        //rectangles
        public Rectangle[] CollisionBoxes = new Rectangle[4];
        public Rectangle all = new Rectangle();


        /// <summary>
        /// This is the constructor
        /// </summary>
        public Paddle(Game game)
            : base(game)
        {
            //Do nothing special in here
        }


        /// <summary>
        /// Allows the game component to perform any initialization it needs to before starting
        /// to run.  This is where it can query for any required services and load content.
        /// </summary>
        public override void Initialize()
        {           
            base.Initialize(); 

            //Initialize the graphics manager
            graphics = GraphicsDevice;

            // Create a new SpriteBatch, which can be used to draw textures.
            spriteBatch = new SpriteBatch(GraphicsDevice);

            //Setting up the rectangles
            //the rectangle that contains the entire paddle
            all.Width = _skin.Width;
            all.Height = _skin.Height;

            //set up the boxes
            SetCollisionBoxes();

            // Set up the _spritePosition
            _spritePosition.Y = (graphics.Viewport.Height / 2) + (_skin.Height / 2);
            if (_RightOrLeft == 1)
                _spritePosition.X = 30.0f;
            else
                _spritePosition.X = graphics.Viewport.Width - (30.0f + _skin.Width);

            // Set up the oldState
            KeyboardState oldState = Keyboard.GetState();

        }

        /// <summary>
        /// This private function will check for keyboard inputs, 
        /// then move a paddle in the respected direction.
        /// This version is for the right paddle
        /// </summary>
        private void UpdateInput()
        {
            // Create a keyboard state to get a new status
            KeyboardState newState = Keyboard.GetState();

            // Is the UP key down?
            if (newState.IsKeyDown(Keys.Up) && _spritePosition.Y > 0)
            {
                // If not down last update, key has just been pressed.
                if (!oldState.IsKeyDown(Keys.Up))
                {
                    _spriteSpeed.Y = -50;
                }
            }
            else if (oldState.IsKeyDown(Keys.Up))
            {
                // Key was down last update, but not down now, so
                // it has just been released.
                _spriteSpeed.Y = 0;
            }

            // Is the DOWN key down?
            if (newState.IsKeyDown(Keys.Down) && _spritePosition.Y < (graphics.Viewport.Height - _skin.Height))
            {
                // If not down last update, key has just been pressed.
                if (!oldState.IsKeyDown(Keys.Down))
                {
                    _spriteSpeed.Y = 50;
                }
            }
            else if (oldState.IsKeyDown(Keys.Down))
            {
                // Key was down last update, but not down now, so
                // it has just been released.
                _spriteSpeed.Y = 0;
            }
            oldState = newState;
        }

        /// <summary>
        /// This private function will check for keyboard inputs, 
        /// then move a paddle in the respected direction.
        /// This version is for the left paddle
        /// </summary>
        private void UpdateInput2()
        {
            // Create a keyboard state to get a new status
            KeyboardState newState = Keyboard.GetState();

            // Is the UP key down?
            if (newState.IsKeyDown(Keys.W) && _spritePosition.Y > 0)
            {
                // If not down last update, key has just been pressed.
                if (!oldState.IsKeyDown(Keys.W))
                {
                    _spriteSpeed.Y = -50;
                }
            }
            else if (oldState.IsKeyDown(Keys.W))
            {
                // Key was down last update, but not down now, so
                // it has just been released.
                _spriteSpeed.Y = 0;
            }

            // Is the DOWN key down?
            if (newState.IsKeyDown(Keys.S) && _spritePosition.Y < (graphics.Viewport.Height - _skin.Height))
            {
                // If not down last update, key has just been pressed.
                if (!oldState.IsKeyDown(Keys.S))
                {
                    _spriteSpeed.Y = 50;
                }
            }
            else if (oldState.IsKeyDown(Keys.S))
            {
                // Key was down last update, but not down now, so
                // it has just been released.
                _spriteSpeed.Y = 0;
            }
            oldState = newState;
        }

        /// <summary>
        /// Allows the game component to update itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        public override void Update(GameTime gameTime)
        {
            if (enabled)
            {
                //reset the "all" rectangle
                all.X = (int)_spritePosition.X;
                all.Y = (int)_spritePosition.Y;

                //reset the collision boxes
                SetCollisionBoxes();

                //move the paddle
                _spritePosition += _spriteSpeed * (float)0.05;

                // Check for user input directing Sprite
                if (_RightOrLeft == 0)
                    UpdateInput();
                else
                    UpdateInput2();
            }
            else
                _spriteSpeed *= 0;

            base.Update(gameTime);
        }

        private void SetCollisionBoxes()
        {
            //the rectangle that contains the top edge of the paddle
            CollisionBoxes[0].Width = all.Width;
            CollisionBoxes[0].Height = 1;
            CollisionBoxes[0].X = all.X;
            CollisionBoxes[0].Y = all.Y;

            //the rectangle that contains the upper half's edge of the paddle
            CollisionBoxes[1].Width = 1;
            CollisionBoxes[1].Height = (all.Height / 2) - 1;
            CollisionBoxes[1].X = all.X + all.Width * _RightOrLeft;
            CollisionBoxes[1].Y = all.Y + 1;

            //the rectangle that contains the lower half's edge of the paddle
            CollisionBoxes[2].Width = 1;
            CollisionBoxes[2].Height = (all.Height / 2) - 1;
            CollisionBoxes[2].X = all.X + all.Width * _RightOrLeft;
            CollisionBoxes[2].Y = all.Y + (all.Height / 2);

            //the rectangle that contains the bottom edge of the paddle
            CollisionBoxes[3].Width = all.Width;
            CollisionBoxes[3].Height = 1;
            CollisionBoxes[3].X = all.X;
            CollisionBoxes[3].Y = all.Y + all.Height;
        }

        /// <summary>
        /// This is called when the game should draw itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        public override void Draw(GameTime gameTime)
        {
            // Draw everything.
            spriteBatch.Begin(SpriteBlendMode.AlphaBlend);
            if (_RightOrLeft == 0)
                spriteBatch.Draw(_skin, _spritePosition, null, new Color(255, 255, 255, (byte)180),
                    0, new Vector2(0, 0), 1.0f, SpriteEffects.FlipHorizontally, 1.0f);
            else 
                spriteBatch.Draw(_skin, _spritePosition, new Color(120, 255, 255, (byte)180));
            spriteBatch.End();

            base.Draw(gameTime);
        }
    }
}